==> thx_api-0.3.10-py3-none-any.whl: thx/api/views.py <==
# -*- coding: utf-8 -*-

"""
View Mixins
============

Per rendere più strutturate le nostre View, e per renderla più omogenee sono
stati creati dei mixin con campi e metodi (vedi in particolare :class:`QuerysetMixin` e :class:`PermissionListView`),
oppure solo come superclassi per un utilizzo futuro:

* :class:`QuerysetMixin`
* :class:`StatusUpdateViewMixin`
* :class:`PermissionListView`
* :class:`ThuxListViewMixin`
* :class:`ThuxRetrieveViewMixin`
* :class:`ThuxCreateViewMixin`
* :class:`ThuxUpdateViewMixin`
* :class:`ThuxDestroyViewMixin`
* :class:`ThuxStatusViewMixin`
* :class:`ThuxViewMixin`

Esempio
--------

::
  from thx.api.views import ThuxViewMixin

  class TemplateViewSet(ThuxViewMixin, viewsets.ModelViewSet):
      queryset = models.Template.objects.all()
      serializer_class = serializers.TemplateSerializer

      def check_permissions(self, request):
        ....

Classi
========

.. autoclass:: QuerysetMixin
   :members:

.. autoclass:: StatusUpdateViewMixin
   :members:

.. autoclass:: PermissionListView
   :members:

.. autoclass:: ThuxListViewMixin
   :members:

.. autoclass:: ThuxRetrieveViewMixin
   :members:

.. autoclass:: ThuxCreateViewMixin
   :members:

.. autoclass:: ThuxUpdateViewMixin
   :members:

.. autoclass:: ThuxDestroyViewMixin
   :members:

.. autoclass:: ThuxStatusViewMixin
   :members:

.. autoclass:: ThuxViewMixin
   :members:

"""

import warnings
from django.contrib.auth.models import Permission
from django.core.exceptions import FieldError, ValidationError
from django.db.models import Q
from django.http import Http404
from django.utils.decorators import classonlymethod

from rest_framework import generics
from rest_framework.response import Response

from . import serializers, exceptions
from .deprecation import RemovedInNextThxVersionWarning


class ThxRemoveViewWarningMixin(RemovedInNextThxVersionWarning):
    @classonlymethod
    def as_view(cls, *args, **kwargs):
        for i in cls.__mro__:
            if i.__module__ == "thx.api.views":
                new_cls_name = i.__name__.replace("ViewMixin", "ModelMixin")
                new_cls_module = i.__module__.replace("views", "mixins")
                warnings.warn(
                    f"The class {i.__name__} in module {i.__module__} is deprecated. "
                    f"Please update your code using the class {new_cls_name} in module {new_cls_module}. ",
                    category=cls,
                    stacklevel=2,
                )
                break
        return super().as_view(*args, **kwargs)


class QuerysetMixin(ThxRemoveViewWarningMixin):
    """
    **QuerysetMixin** è una classe che aggiunge funzionalità per manipolare i queryset impostando query_params precisi
    nella chiamata; ogni metodo gestisce una tipologia di filtri:

    * :meth:`get_filters`
    * :meth:`get_excludes`
    * :meth:`get_order_by`

    Entrambe le FK sono definite on_delete=models.PROTECT
    """
    def get_filters(self):
        """
        costruisce i filtri per il queryset leggendo il querystring e cercando il seguente pattern::

            filter__[dj_filter]=[value]

        Questa sintassi indica che il queryset dovrà essere filtrato con `dj_filter` e con valore `value`::

        /api/.../template/?filter__typology__code=flow&filter__status__lte=1

        che si traduce in questo filtro::

            filters = {
                "typology__code": "flow",
                "status__lte": 1
             }
            queryset = queryset.filter(**filters)


        può essere utilizzato più volte nella stessa url
        """
        filters = dict()
        for key, value in self.request.GET.items():
            if key.startswith('filter__'):
                key = key[8:]
                if key.endswith('isnull'):
                    if value.lower() == 'true':
                        filters[key] = True
                    elif value.lower() == 'false':
                        filters[key] = False
                    else:
                        filters[key] = value
                elif key.endswith('__in'):
                    filters[key] = value.split(",")
                else:
                    filters[key] = value
        return filters

    def get_excludes(self):
        """
        costruisce gli `exclude` per il queryset leggendo il querystring e cercando il seguente pattern::

            exclude__[dj_exclude]=[value]

        Questa sintassi indica che al queryset dovranno essere applicati i `dj_exclude` con valore `value`::

        /api/.../template/?exclude__typology__code=flow&exclude__status__lte=1

        che si traduce in questo filtro::

            queryset = queryset.exclude(typology__code="flow").exclude(status__lte=1)


        può essere utilizzato più volte nella stessa url
        """
        excludes = dict()
        for key, value in self.request.GET.items():
            if key.startswith('exclude__'):
                key = key[9:]
                if key.endswith('isnull'):
                    if value.lower() == 'true':
                        excludes[key] = True
                    elif value.lower() == 'false':
                        excludes[key] = False
                    else:
                        excludes[key] = value
                elif key.endswith('__in'):
                    excludes[key] = value.split(",")
                else:
                    excludes[key] = value
        return excludes

    def get_order_by(self):
        """
        costruisce l'order by per il queryset leggendo il querystring e cercando il seguente pattern::

            order_by=[order1.order2.-order3]

        Questa sintassi indica che il queryset dovrà essere ordinato con order1, order2 `dj_exclude`::

        /api/.../template/?order_by=order1.order2.-order3

        che si traduce in questo order_by::

            queryset = queryset.order_by("order1", "order2", "-order3")

        """
        order_by = list()
        for key, value in self.request.GET.items():
            if key.startswith('order_by'):
                order_by = value.split('.')
        return order_by

    def get_queryset(self):
        """
        utilizza in cascata i metodi get_filters(), get_excludes(), get_order_by() e li applica al queryset
        """
        filters = self.get_filters()
        excludes = self.get_excludes()
        order_by = self.get_order_by()
        queryset = super(QuerysetMixin, self).get_queryset()
        try:
            if filters:
                queryset = queryset.filter(**filters)
            if excludes:
                for key, value in excludes.items():
                    queryset = queryset.exclude(**{key: value})
            if order_by:
                queryset = queryset.order_by(*order_by)
        except ValueError:
            # just a little philosophy but it may be correct to raise a 400 bad request
            # without information
            raise Http404
        except FieldError:
            # TODO migliorare messaggio di errore
            raise exceptions.BadRequest
        except ValidationError as e:
            raise exceptions.BadRequest(detail='\n'.join(e.messages))
        return queryset.distinct()

    @property
    def paginator(self):
        """
        fa si che la response della vista non sarà paginata, leggendo il querystring e cercando il seguente pattern::

            no_page=no_page

            /api/.../template/?no_page=no_page

        """
        if self.request.query_params.get('no_page'):
            self.pagination_class = None
        return super(QuerysetMixin, self).paginator

    # TODO chiedere a chi ha scritto il metodo
    def paginated_list(self, request, custom_qs=None, *args, **kwargs):
        """
        :param custom_qs: non standard queryset (example .values() or .only() Or .defer())
        :return: paginated Response
        """
        if isinstance(custom_qs, type(None)):
            return super().list(request)

        queryset = self.filter_queryset(queryset=custom_qs)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)


class StatusUpdateViewMixin(ThxRemoveViewWarningMixin):
    """
    **StatusUpdateViewMixin** è un mixin che riscrive il metodo put per impostare lo status del modello con il valore
    inviato nel payload; solitamente i valori sono 1 o 0 (enabled o disabled)
    """

    def put(self, request, *args, **kwargs):
        instance = self.get_object()
        instance.status = self.new_status
        instance.save()
        serializer = self.get_serializer(instance)
        return Response(serializer.data)


class PermissionListView(generics.ListAPIView):
    """
    **PermissionListView** è una vista che riscrive il metodo get_queryset e si occupa di restituire tutti i permessi
    associati all'utente autenticato
    """

    serializer_class = serializers.PermissionSerializer
    pagination_class = None

    def get_queryset(self):
        user = self.request.user
        qs = Permission.objects.all()
        if not user.is_superuser:
            qs = qs.filter(Q(group__user=user) | Q(user=user))

        if 'app_label' in self.kwargs:
            qs = qs.filter(content_type__app_label=self.kwargs['app_label'])
            if 'model_name' in self.kwargs:
                qs = qs.filter(content_type__model=self.kwargs['model_name'])

        return qs


class ThuxListViewMixin(QuerysetMixin):
    """
    **ThuxListViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    pass


class ThuxRetrieveViewMixin(ThxRemoveViewWarningMixin):
    """
    **ThuxRetrieveViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    pass


class ThuxCreateViewMixin(ThxRemoveViewWarningMixin):
    """
    **ThuxCreateViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user = None


class ThuxUpdateViewMixin(ThxRemoveViewWarningMixin):
    """
    **ThuxUpdateViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user = None


class ThuxDestroyViewMixin(ThxRemoveViewWarningMixin):
    """
    **ThuxDestroyViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    pass


class ThuxStatusViewMixin(ThxRemoveViewWarningMixin):
    """
    **ThuxStatusViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user = None


class ThuxViewMixin(
    ThuxListViewMixin, ThuxRetrieveViewMixin, ThuxCreateViewMixin, ThuxUpdateViewMixin, ThuxDestroyViewMixin
):
    """
    **ThuxViewMixin** è un mixin che importa  in blocco i mixin
    * :class:`ThuxListViewMixin`
    * :class:`ThuxRetrieveViewMixin`
    * :class:`ThuxCreateViewMixin`
    * :class:`ThuxUpdateViewMixin`
    * :class:`ThuxDestroyViewMixin`
    * :class:`ThuxStatusViewMixin`
    """

    pass

==> thx_api-0.3.12-py3-none-any.whl: thx/api/views.py <==
# -*- coding: utf-8 -*-

"""
View Mixins
============

Per rendere più strutturate le nostre View, e per renderla più omogenee sono
stati creati dei mixin con campi e metodi (vedi in particolare :class:`QuerysetMixin` e :class:`PermissionListView`),
oppure solo come superclassi per un utilizzo futuro:

* :class:`QuerysetMixin`
* :class:`StatusUpdateViewMixin`
* :class:`PermissionListView`
* :class:`ThuxListViewMixin`
* :class:`ThuxRetrieveViewMixin`
* :class:`ThuxCreateViewMixin`
* :class:`ThuxUpdateViewMixin`
* :class:`ThuxDestroyViewMixin`
* :class:`ThuxStatusViewMixin`
* :class:`ThuxViewMixin`

Esempio
--------

::
  from thx.api.views import ThuxViewMixin

  class TemplateViewSet(ThuxViewMixin, viewsets.ModelViewSet):
      queryset = models.Template.objects.all()
      serializer_class = serializers.TemplateSerializer

      def check_permissions(self, request):
        ....

Classi
========

.. autoclass:: QuerysetMixin
   :members:

.. autoclass:: StatusUpdateViewMixin
   :members:

.. autoclass:: PermissionListView
   :members:

.. autoclass:: ThuxListViewMixin
   :members:

.. autoclass:: ThuxRetrieveViewMixin
   :members:

.. autoclass:: ThuxCreateViewMixin
   :members:

.. autoclass:: ThuxUpdateViewMixin
   :members:

.. autoclass:: ThuxDestroyViewMixin
   :members:

.. autoclass:: ThuxStatusViewMixin
   :members:

.. autoclass:: ThuxViewMixin
   :members:

"""

import warnings
from django.contrib.auth.models import Permission
from django.core.exceptions import FieldError, ValidationError
from django.db.models import Q
from django.http import Http404
from django.utils.decorators import classonlymethod

from rest_framework import generics
from rest_framework.response import Response

from . import serializers, exceptions
from .deprecation import RemovedInNextThxVersionWarning


class ThxRemoveViewWarningMixin:
    @classonlymethod
    def as_view(cls, *args, **kwargs):
        for i in cls.__mro__:
            if i.__module__ == "thx.api.views":
                new_cls_name = i.__name__.replace("ViewMixin", "ModelMixin")
                new_cls_module = i.__module__.replace("views", "mixins")
                warnings.warn(
                    f"The class {i.__name__} in module {i.__module__} is deprecated. "
                    f"Please update your code using the class {new_cls_name} in module {new_cls_module}. ",
                    category=RemovedInNextThxVersionWarning,
                    stacklevel=2,
                )
                break
        return super().as_view(*args, **kwargs)


class QuerysetMixin(ThxRemoveViewWarningMixin):
    """
    **QuerysetMixin** è una classe che aggiunge funzionalità per manipolare i queryset impostando query_params precisi
    nella chiamata; ogni metodo gestisce una tipologia di filtri:

    * :meth:`get_filters`
    * :meth:`get_excludes`
    * :meth:`get_order_by`

    Entrambe le FK sono definite on_delete=models.PROTECT
    """
    def get_filters(self):
        """
        costruisce i filtri per il queryset leggendo il querystring e cercando il seguente pattern::

            filter__[dj_filter]=[value]

        Questa sintassi indica che il queryset dovrà essere filtrato con `dj_filter` e con valore `value`::

        /api/.../template/?filter__typology__code=flow&filter__status__lte=1

        che si traduce in questo filtro::

            filters = {
                "typology__code": "flow",
                "status__lte": 1
             }
            queryset = queryset.filter(**filters)


        può essere utilizzato più volte nella stessa url
        """
        filters = dict()
        for key, value in self.request.GET.items():
            if key.startswith('filter__'):
                key = key[8:]
                if key.endswith('isnull'):
                    if value.lower() == 'true':
                        filters[key] = True
                    elif value.lower() == 'false':
                        filters[key] = False
                    else:
                        filters[key] = value
                elif key.endswith('__in'):
                    filters[key] = value.split(",")
                else:
                    filters[key] = value
        return filters

    def get_excludes(self):
        """
        costruisce gli `exclude` per il queryset leggendo il querystring e cercando il seguente pattern::

            exclude__[dj_exclude]=[value]

        Questa sintassi indica che al queryset dovranno essere applicati i `dj_exclude` con valore `value`::

        /api/.../template/?exclude__typology__code=flow&exclude__status__lte=1

        che si traduce in questo filtro::

            queryset = queryset.exclude(typology__code="flow").exclude(status__lte=1)


        può essere utilizzato più volte nella stessa url
        """
        excludes = dict()
        for key, value in self.request.GET.items():
            if key.startswith('exclude__'):
                key = key[9:]
                if key.endswith('isnull'):
                    if value.lower() == 'true':
                        excludes[key] = True
                    elif value.lower() == 'false':
                        excludes[key] = False
                    else:
                        excludes[key] = value
                elif key.endswith('__in'):
                    excludes[key] = value.split(",")
                else:
                    excludes[key] = value
        return excludes

    def get_order_by(self):
        """
        costruisce l'order by per il queryset leggendo il querystring e cercando il seguente pattern::

            order_by=[order1.order2.-order3]

        Questa sintassi indica che il queryset dovrà essere ordinato con order1, order2 `dj_exclude`::

        /api/.../template/?order_by=order1.order2.-order3

        che si traduce in questo order_by::

            queryset = queryset.order_by("order1", "order2", "-order3")

        """
        order_by = list()
        for key, value in self.request.GET.items():
            if key.startswith('order_by'):
                order_by = value.split('.')
        return order_by

    def get_queryset(self):
        """
        utilizza in cascata i metodi get_filters(), get_excludes(), get_order_by() e li applica al queryset
        """
        filters = self.get_filters()
        excludes = self.get_excludes()
        order_by = self.get_order_by()
        queryset = super(QuerysetMixin, self).get_queryset()
        try:
            if filters:
                queryset = queryset.filter(**filters)
            if excludes:
                for key, value in excludes.items():
                    queryset = queryset.exclude(**{key: value})
            if order_by:
                queryset = queryset.order_by(*order_by)
        except ValueError:
            # just a little philosophy but it may be correct to raise a 400 bad request
            # without information
            raise Http404
        except FieldError:
            # TODO migliorare messaggio di errore
            raise exceptions.BadRequest
        except ValidationError as e:
            raise exceptions.BadRequest(detail='\n'.join(e.messages))
        return queryset.distinct()

    @property
    def paginator(self):
        """
        fa si che la response della vista non sarà paginata, leggendo il querystring e cercando il seguente pattern::

            no_page=no_page

            /api/.../template/?no_page=no_page

        """
        if self.request.query_params.get('no_page'):
            self.pagination_class = None
        return super(QuerysetMixin, self).paginator

    # TODO chiedere a chi ha scritto il metodo
    def paginated_list(self, request, custom_qs=None, *args, **kwargs):
        """
        :param custom_qs: non standard queryset (example .values() or .only() Or .defer())
        :return: paginated Response
        """
        if isinstance(custom_qs, type(None)):
            return super().list(request)

        queryset = self.filter_queryset(queryset=custom_qs)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)


class StatusUpdateViewMixin(ThxRemoveViewWarningMixin):
    """
    **StatusUpdateViewMixin** è un mixin che riscrive il metodo put per impostare lo status del modello con il valore
    inviato nel payload; solitamente i valori sono 1 o 0 (enabled o disabled)
    """

    def put(self, request, *args, **kwargs):
        instance = self.get_object()
        instance.status = self.new_status
        instance.save()
        serializer = self.get_serializer(instance)
        return Response(serializer.data)


class PermissionListView(generics.ListAPIView):
    """
    **PermissionListView** è una vista che riscrive il metodo get_queryset e si occupa di restituire tutti i permessi
    associati all'utente autenticato
    """

    serializer_class = serializers.PermissionSerializer
    pagination_class = None

    def get_queryset(self):
        user = self.request.user
        qs = Permission.objects.all()
        if not user.is_superuser:
            qs = qs.filter(Q(group__user=user) | Q(user=user))

        if 'app_label' in self.kwargs:
            qs = qs.filter(content_type__app_label=self.kwargs['app_label'])
            if 'model_name' in self.kwargs:
                qs = qs.filter(content_type__model=self.kwargs['model_name'])

        return qs


class ThuxListViewMixin(QuerysetMixin):
    """
    **ThuxListViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    pass


class ThuxRetrieveViewMixin(ThxRemoveViewWarningMixin):
    """
    **ThuxRetrieveViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    pass


class ThuxCreateViewMixin(ThxRemoveViewWarningMixin):
    """
    **ThuxCreateViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user = None


class ThuxUpdateViewMixin(ThxRemoveViewWarningMixin):
    """
    **ThuxUpdateViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user = None


class ThuxDestroyViewMixin(ThxRemoveViewWarningMixin):
    """
    **ThuxDestroyViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    pass


class ThuxStatusViewMixin(ThxRemoveViewWarningMixin):
    """
    **ThuxStatusViewMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user = None


class ThuxViewMixin(
    ThuxListViewMixin, ThuxRetrieveViewMixin, ThuxCreateViewMixin, ThuxUpdateViewMixin, ThuxDestroyViewMixin
):
    """
    **ThuxViewMixin** è un mixin che importa  in blocco i mixin
    * :class:`ThuxListViewMixin`
    * :class:`ThuxRetrieveViewMixin`
    * :class:`ThuxCreateViewMixin`
    * :class:`ThuxUpdateViewMixin`
    * :class:`ThuxDestroyViewMixin`
    * :class:`ThuxStatusViewMixin`
    """

    pass
